/*********************************************************************
 *
 * Copyright (c) 1998
 * 3Com/Palm Computing Division.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *		This product includes software developed by 3Com and its
 *		contributors.
 *
 * 4. Neither 3Com nor the names of its contributors may be used to
 *    endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE 3COM AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 3COM OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 *******************************************************************/

#ifndef _HOSTCONTROL_H_
#define _HOSTCONTROL_H_

#include <SysTraps.h>


/*
	Set the base value for selectors. Note that this value MUST be
	two bytes long and have the high byte be non-zero. The reason
	for this has to do with the way SysGremlins was originally
	declared. It took a GremlinsSelector enumerated value. Originally,
	there was only one value, and it was zero. The way the 68K compiler
	works, it decides that GremlinsSelectors are only one byte long,
	so a call to SysGremlins would push one byte onto the stack. Because
	all values on the stack need to be word-aligned, the processor
	subtracts 1 from the stack before pushing on the byte. Therefore,
	the stack looks like:

			previous contents
			garbage byte
			selector
			return address

	With this setup, we have two choices: leave the selector size at
	one byte and limit ourselves to 256 functions, or define the selector
	to be a two byte value, with the first 256 values (all those with 0x00
	in the upper byte) to be GremlinIsOn. The latter sounds preferable, so
	we start the new selectors at 0x0100.
*/

#define hostSelectorBase	0x0100
#define hostSelectorChunk	0x0100
#define hostSection(x)		(hostSelectorBase + (hostSelectorChunk * (x)))

typedef enum
{
	// Host information selectors

	hostSelectorGetHostVersion = hostSection(0),
	hostSelectorGetHostID,
	hostSelectorGetHostPlatform,
	hostSelectorIsSelectorImplemented,
	hostSelectorGestalt,

	// Profiler selectors

	hostSelectorProfileInit = hostSection(1),
	hostSelectorProfileStart,
	hostSelectorProfileStop,
	hostSelectorProfileDump,
	hostSelectorProfileCleanup,

	// Std C Library wrapper selectors

	hostSelectorErrNo = hostSection(2),

	hostSelectorFClose,
	hostSelectorFEOF,
	hostSelectorFError,
	hostSelectorFFlush,
	hostSelectorFGetC,
	hostSelectorFGetPos,
	hostSelectorFGetS,
	hostSelectorFOpen,
	hostSelectorFPrintF,		// Floating point not yet supported in Poser
	hostSelectorFPutC,
	hostSelectorFPutS,
	hostSelectorFRead,
	hostSelectorRemove,			// Not yet implemented in Poser
	hostSelectorRename,			// Not yet implemented in Poser
	hostSelectorFReopen,		// Not yet implemented in Poser
	hostSelectorFScanF,			// Not yet implemented
	hostSelectorFSeek,
	hostSelectorFSetPos,
	hostSelectorFTell,
	hostSelectorFWrite,
	hostSelectorTmpFile,
	hostSelectorTmpNam,			// Not yet implemented in Poser
	hostSelectorGetEnv,

	hostSelectorMalloc,			// Not yet implemented in Poser
	hostSelectorRealloc,		// Not yet implemented in Poser
	hostSelectorFree,			// Not yet implemented in Poser
	
	// Gremlin selectors

	hostSelectorGremlinIsRunning = hostSection(3),
	hostSelectorGremlinNumber,
	hostSelectorGremlinCounter,
	hostSelectorGremlinLimit,
//	hostSelectorGremlinApp,

	// Database selectors

	hostSelectorImportFile = hostSection(4),
	hostSelectorExportFile,

	// Preferences selectors

	hostSelectorSetDebugOption = hostSection(5),		// Not yet implemented
	hostSelectorSetWindowScale,							// Not yet implemented
	hostSelectorSetWindowBackground,					// Not yet implemented

	// Logging selectors

	hostSelectorLogFile = hostSection(6),
	hostSelectorSetLogFileSize,

	hostSelectorLastTrapNumber
} HostControlTrapNumber;


typedef struct HostFILE
{
	long	_field;
} HostFILE;



typedef long	HostBool;
typedef long	HostErr;
typedef long	HostID;
typedef long	HostPlatform;
typedef long	HostBG;

#ifndef hostErrorClass
	#define	hostErrorClass				0x1C00	// Host Control Manager
#else
	#if hostErrorClass != 0x1C00
		#error "You cannot change hostErrorClass without telling us."
	#endif
#endif

enum	// HostErr values
{
	hostErrNone = 0,

	hostErrBase = hostErrorClass,

	hostErrUnknownGestaltSelector,

	hostErrDiskError,

	hostErrOutOfMemory,
	hostErrMemReadOutOfRange,
	hostErrMemWriteOutOfRange,
	hostErrMemInvalidPtr,

	hostErrInvalidParameter
};


enum	// HostID values
{
	hostIDPalmOS,
	hostIDPalmOSEmulator,	// The Copilot thingy
	hostIDPalmOSSimulator	// The Mac libraries you link with thingy
};


enum	// HostPlatform values
{
	hostPlatformPalmOS,
	hostPlatformWindows,
	hostPlatformMacintosh,
	hostPlatformUnix
};


enum	// HostBG values
{
	hostBGGray,
	hostBGWhite
};


// Define this in case SysTraps.h doesn't have it.

#define sysTrapHostControl (sysTrapBase + 836)

#define HOST_TRAP(selector)													\
	FIVEWORD_INLINE(														\
		0x3F3C, selector,					/* MOVE.W #selector, -(A7)	*/	\
		m68kTrapInstr + sysDispatchTrapNum,	/* TRAP $F					*/	\
		sysTrapHostControl,					/* sysTrapHostControl		*/	\
		0x544F)								/* ADD.Q #2, A7				*/ 


#ifdef __GNUC__
#define HostGetHostVersion() gcc_HostGetHostVersion(hostSelectorGetHostVersion)
long gcc_HostGetHostVersion(Word) SYS_TRAP(sysTrapHostControl);
#else
long			HostGetHostVersion			(void)										HOST_TRAP (hostSelectorGetHostVersion);
#endif
#ifdef __GNUC__
#define HostGetHostID() gcc_HostGetHostID(hostSelectorGetHostID)
HostID gcc_HostGetHostID(Word) SYS_TRAP(sysTrapHostControl);
#else
HostID			HostGetHostID				(void)										HOST_TRAP (hostSelectorGetHostID);
#endif
#ifdef __GNUC__
#define HostGetHostPlatform() gcc_HostGetHostPlatform(hostSelectorGetHostPlatform)
HostPlatform gcc_HostGetHostPlatform(Word) SYS_TRAP(sysTrapHostControl);
#else
HostPlatform	HostGetHostPlatform			(void)										HOST_TRAP (hostSelectorGetHostPlatform);
#endif
#ifdef __GNUC__
#define HostIsSelectorImplemented(args...) gcc_HostIsSelectorImplemented(hostSelectorIsSelectorImplemented, ## args)
HostBool gcc_HostIsSelectorImplemented(Word, long selector) SYS_TRAP(sysTrapHostControl);
#else
HostBool		HostIsSelectorImplemented	(long selector)								HOST_TRAP (hostSelectorIsSelectorImplemented);
#endif
#ifdef __GNUC__
#define HostGestalt(args...) gcc_HostGestalt(hostSelectorGestalt, ## args)
HostErr gcc_HostGestalt(Word, long gestSel, long* response) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostGestalt					(long gestSel, long* response)				HOST_TRAP (hostSelectorGestalt);
#endif

#ifdef __GNUC__
#define HostProfileInit(args...) gcc_HostProfileInit(hostSelectorProfileInit, ## args)
HostErr gcc_HostProfileInit(Word, long maxCalls, long maxDepth) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostProfileInit				(long maxCalls, long maxDepth)				HOST_TRAP (hostSelectorProfileInit);
#endif
#ifdef __GNUC__
#define HostProfileStart() gcc_HostProfileStart(hostSelectorProfileStart)
HostErr gcc_HostProfileStart(Word) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostProfileStart			(void)										HOST_TRAP (hostSelectorProfileStart);
#endif
#ifdef __GNUC__
#define HostProfileStop() gcc_HostProfileStop(hostSelectorProfileStop)
HostErr gcc_HostProfileStop(Word) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostProfileStop				(void)										HOST_TRAP (hostSelectorProfileStop);
#endif
#ifdef __GNUC__
#define HostProfileDump(args...) gcc_HostProfileDump(hostSelectorProfileDump, ## args)
HostErr gcc_HostProfileDump(Word, const char* filename) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostProfileDump				(const char* filename)						HOST_TRAP (hostSelectorProfileDump);
#endif
#ifdef __GNUC__
#define HostProfileCleanup() gcc_HostProfileCleanup(hostSelectorProfileCleanup)
HostErr gcc_HostProfileCleanup(Word) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostProfileCleanup			(void)										HOST_TRAP (hostSelectorProfileCleanup);
#endif

#ifdef __GNUC__
#define HostErrNo() gcc_HostErrNo(hostSelectorErrNo)
long gcc_HostErrNo(Word) SYS_TRAP(sysTrapHostControl);
#else
long			HostErrNo					(void)										HOST_TRAP (hostSelectorErrNo);
#endif

#ifdef __GNUC__
#define HostFClose(args...) gcc_HostFClose(hostSelectorFClose, ## args)
long gcc_HostFClose(Word, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFClose					(HostFILE* f)								HOST_TRAP (hostSelectorFClose);
#endif
#ifdef __GNUC__
#define HostFEOF(args...) gcc_HostFEOF(hostSelectorFEOF, ## args)
long gcc_HostFEOF(Word, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFEOF					(HostFILE* f)								HOST_TRAP (hostSelectorFEOF);
#endif
#ifdef __GNUC__
#define HostFError(args...) gcc_HostFError(hostSelectorFError, ## args)
long gcc_HostFError(Word, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFError					(HostFILE* f)								HOST_TRAP (hostSelectorFError);
#endif
#ifdef __GNUC__
#define HostFFlush(args...) gcc_HostFFlush(hostSelectorFFlush, ## args)
long gcc_HostFFlush(Word, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFFlush					(HostFILE* f)								HOST_TRAP (hostSelectorFFlush);
#endif
#ifdef __GNUC__
#define HostFGetC(args...) gcc_HostFGetC(hostSelectorFGetC, ## args)
long gcc_HostFGetC(Word, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFGetC					(HostFILE* f)								HOST_TRAP (hostSelectorFGetC);
#endif
#ifdef __GNUC__
#define HostFGetPos(args...) gcc_HostFGetPos(hostSelectorFGetPos, ## args)
long gcc_HostFGetPos(Word, HostFILE* f, long* posP) SYS_TRAP(sysTrapHostControl);
#else
long			HostFGetPos					(HostFILE* f, long* posP)					HOST_TRAP (hostSelectorFGetPos);
#endif
#ifdef __GNUC__
#define HostFGetS(args...) gcc_HostFGetS(hostSelectorFGetS, ## args)
char* gcc_HostFGetS(Word, char* s, long n, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
char*			HostFGetS					(char* s, long n, HostFILE* f)				HOST_TRAP (hostSelectorFGetS);
#endif
#ifdef __GNUC__
#define HostFOpen(args...) gcc_HostFOpen(hostSelectorFOpen, ## args)
HostFILE* gcc_HostFOpen(Word, const char* name, const char* mode) SYS_TRAP(sysTrapHostControl);
#else
HostFILE*		HostFOpen					(const char* name, const char* mode)		HOST_TRAP (hostSelectorFOpen);
#endif
#ifdef __GNUC__
#define HostFPrintF(args...) gcc_HostFPrintF(hostSelectorFPrintF, ## args)
long gcc_HostFPrintF(Word, HostFILE* f, const char* fmt, ...) SYS_TRAP(sysTrapHostControl);
#else
long			HostFPrintF					(HostFILE* f, const char* fmt, ...)			HOST_TRAP (hostSelectorFPrintF);
#endif
#ifdef __GNUC__
#define HostFPutC(args...) gcc_HostFPutC(hostSelectorFPutC, ## args)
long gcc_HostFPutC(Word, long c, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFPutC					(long c, HostFILE* f)						HOST_TRAP (hostSelectorFPutC);
#endif
#ifdef __GNUC__
#define HostFPutS(args...) gcc_HostFPutS(hostSelectorFPutS, ## args)
long gcc_HostFPutS(Word, const char* s, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFPutS					(const char* s, HostFILE* f)				HOST_TRAP (hostSelectorFPutS);
#endif
#ifdef __GNUC__
#define HostFRead(args...) gcc_HostFRead(hostSelectorFRead, ## args)
long gcc_HostFRead(Word, void* buffer, long size, long count, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFRead					(void* buffer, long size, long count, HostFILE* f)
																						HOST_TRAP (hostSelectorFRead);
#endif
#ifdef __GNUC__
#define HostRemove(args...) gcc_HostRemove(hostSelectorRemove, ## args)
long gcc_HostRemove(Word, const char* name) SYS_TRAP(sysTrapHostControl);
#else
long			HostRemove					(const char* name)							HOST_TRAP (hostSelectorRemove);
#endif
#ifdef __GNUC__
#define HostRename(args...) gcc_HostRename(hostSelectorRename, ## args)
long gcc_HostRename(Word, const char* oldName, const char* newName) SYS_TRAP(sysTrapHostControl);
#else
long			HostRename					(const char* oldName, const char* newName)	HOST_TRAP (hostSelectorRename);
#endif
#ifdef __GNUC__
#define HostFReopen(args...) gcc_HostFReopen(hostSelectorFReopen, ## args)
HostFILE* gcc_HostFReopen(Word, const char* name, const char* mode, HostFILE *f) SYS_TRAP(sysTrapHostControl);
#else
HostFILE*		HostFReopen					(const char* name, const char* mode, HostFILE *f)
																						HOST_TRAP (hostSelectorFReopen);
#endif
#ifdef __GNUC__
#define HostFScanF(args...) gcc_HostFScanF(hostSelectorFScanF, ## args)
long gcc_HostFScanF(Word, HostFILE* f, const char *fmt, ...) SYS_TRAP(sysTrapHostControl);
#else
long			HostFScanF					(HostFILE* f, const char *fmt, ...)			HOST_TRAP(hostSelectorFScanF);
#endif
#ifdef __GNUC__
#define HostFSeek(args...) gcc_HostFSeek(hostSelectorFSeek, ## args)
long gcc_HostFSeek(Word, HostFILE* f, long offset, long origin) SYS_TRAP(sysTrapHostControl);
#else
long			HostFSeek					(HostFILE* f, long offset, long origin)		HOST_TRAP (hostSelectorFSeek);
#endif
#ifdef __GNUC__
#define HostFSetPos(args...) gcc_HostFSetPos(hostSelectorFSetPos, ## args)
long gcc_HostFSetPos(Word, HostFILE* f, long* pos) SYS_TRAP(sysTrapHostControl);
#else
long			HostFSetPos					(HostFILE* f, long* pos)					HOST_TRAP (hostSelectorFSetPos);
#endif
#ifdef __GNUC__
#define HostFTell(args...) gcc_HostFTell(hostSelectorFTell, ## args)
long gcc_HostFTell(Word, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFTell					(HostFILE* f)								HOST_TRAP (hostSelectorFTell);
#endif
#ifdef __GNUC__
#define HostFWrite(args...) gcc_HostFWrite(hostSelectorFWrite, ## args)
long gcc_HostFWrite(Word, const void* buffer, long size, long count, HostFILE* f) SYS_TRAP(sysTrapHostControl);
#else
long			HostFWrite					(const void* buffer, long size, long count, HostFILE* f)
																						HOST_TRAP (hostSelectorFWrite);
#endif
#ifdef __GNUC__
#define HostTmpFile() gcc_HostTmpFile(hostSelectorTmpFile)
HostFILE* gcc_HostTmpFile(Word) SYS_TRAP(sysTrapHostControl);
#else
HostFILE*		HostTmpFile					(void)										HOST_TRAP (hostSelectorTmpFile);
#endif
#ifdef __GNUC__
#define HostTmpNam(args...) gcc_HostTmpNam(hostSelectorTmpNam, ## args)
char* gcc_HostTmpNam(Word, char *name) SYS_TRAP(sysTrapHostControl);
#else
char*			HostTmpNam					(char *name)								HOST_TRAP (hostSelectorTmpNam);
#endif
#ifdef __GNUC__
#define HostGetEnv(args...) gcc_HostGetEnv(hostSelectorGetEnv, ## args)
char* gcc_HostGetEnv(Word, const char*) SYS_TRAP(sysTrapHostControl);
#else
char*			HostGetEnv					(const char*)								HOST_TRAP (hostSelectorGetEnv);
#endif

#ifdef __GNUC__
#define HostMalloc(args...) gcc_HostMalloc(hostSelectorMalloc, ## args)
void* gcc_HostMalloc(Word, long size) SYS_TRAP(sysTrapHostControl);
#else
void*			HostMalloc					(long size)									HOST_TRAP (hostSelectorMalloc);
#endif
#ifdef __GNUC__
#define HostRealloc(args...) gcc_HostRealloc(hostSelectorRealloc, ## args)
void* gcc_HostRealloc(Word, void* p, long size) SYS_TRAP(sysTrapHostControl);
#else
void*			HostRealloc					(void* p, long size)						HOST_TRAP (hostSelectorRealloc);
#endif
#ifdef __GNUC__
#define HostFree(args...) gcc_HostFree(hostSelectorFree, ## args)
void gcc_HostFree(Word, void* p) SYS_TRAP(sysTrapHostControl);
#else
void			HostFree					(void* p)									HOST_TRAP (hostSelectorFree);
#endif

#ifdef __GNUC__
#define HostGremlinIsRunning() gcc_HostGremlinIsRunning(hostSelectorGremlinIsRunning)
HostBool gcc_HostGremlinIsRunning(Word) SYS_TRAP(sysTrapHostControl);
#else
HostBool		HostGremlinIsRunning		(void)										HOST_TRAP (hostSelectorGremlinIsRunning);
#endif
#ifdef __GNUC__
#define HostGremlinNumber() gcc_HostGremlinNumber(hostSelectorGremlinNumber)
long gcc_HostGremlinNumber(Word) SYS_TRAP(sysTrapHostControl);
#else
long			HostGremlinNumber			(void)										HOST_TRAP (hostSelectorGremlinNumber);
#endif
#ifdef __GNUC__
#define HostGremlinCounter() gcc_HostGremlinCounter(hostSelectorGremlinCounter)
long gcc_HostGremlinCounter(Word) SYS_TRAP(sysTrapHostControl);
#else
long			HostGremlinCounter			(void)										HOST_TRAP (hostSelectorGremlinCounter);
#endif
#ifdef __GNUC__
#define HostGremlinLimit() gcc_HostGremlinLimit(hostSelectorGremlinLimit)
long gcc_HostGremlinLimit(Word) SYS_TRAP(sysTrapHostControl);
#else
long			HostGremlinLimit			(void)										HOST_TRAP (hostSelectorGremlinLimit);
#endif

#ifdef __GNUC__
#define HostImportFile(args...) gcc_HostImportFile(hostSelectorImportFile, ## args)
HostErr gcc_HostImportFile(Word, const char* fileName, long cardNum) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostImportFile				(const char* fileName, long cardNum)		HOST_TRAP (hostSelectorImportFile);
#endif
#ifdef __GNUC__
#define HostExportFile(args...) gcc_HostExportFile(hostSelectorExportFile, ## args)
HostErr gcc_HostExportFile(Word, const char* fileName, long cardNum, const char* dbName) SYS_TRAP(sysTrapHostControl);
#else
HostErr			HostExportFile				(const char* fileName, long cardNum, const char* dbName)
																						HOST_TRAP (hostSelectorExportFile);
#endif

#ifdef __GNUC__
#define HostSetDebugOption(args...) gcc_HostSetDebugOption(hostSelectorSetDebugOption, ## args)
HostBool gcc_HostSetDebugOption(Word, HostBool newSetting) SYS_TRAP(sysTrapHostControl);
#else
HostBool		HostSetDebugOption			(HostBool newSetting)						HOST_TRAP (hostSelectorSetDebugOption);
#endif
#ifdef __GNUC__
#define HostSetWindowScale(args...) gcc_HostSetWindowScale(hostSelectorSetWindowScale, ## args)
long gcc_HostSetWindowScale(Word, long newScale) SYS_TRAP(sysTrapHostControl);
#else
long			HostSetWindowScale			(long newScale)								HOST_TRAP (hostSelectorSetWindowScale);
#endif
#ifdef __GNUC__
#define HostSetWindowBackground(args...) gcc_HostSetWindowBackground(hostSelectorSetWindowBackground, ## args)
HostBG gcc_HostSetWindowBackground(Word, HostBG newBackground) SYS_TRAP(sysTrapHostControl);
#else
HostBG			HostSetWindowBackground		(HostBG newBackground)						HOST_TRAP (hostSelectorSetWindowBackground);
#endif

#ifdef __GNUC__
#define HostLogFile() gcc_HostLogFile(hostSelectorLogFile)
HostFILE* gcc_HostLogFile(Word) SYS_TRAP(sysTrapHostControl);
#else
HostFILE*		HostLogFile					(void)										HOST_TRAP (hostSelectorLogFile);
#endif
#ifdef __GNUC__
#define HostSetLogFileSize(args...) gcc_HostSetLogFileSize(hostSelectorSetLogFileSize, ## args)
void gcc_HostSetLogFileSize(Word, long) SYS_TRAP(sysTrapHostControl);
#else
void			HostSetLogFileSize			(long)										HOST_TRAP (hostSelectorSetLogFileSize);
#endif

#endif /* _HOSTCONTROL_H_ */
